Class: Chef::Knife::Ec2FlavorList
- Inherits:
-
Chef::Knife
- Object
- Chef::Knife
- Chef::Knife::Ec2FlavorList
- Includes:
- Ec2Base
- Defined in:
- lib/chef/knife/ec2_flavor_list.rb
Instance Method Summary collapse
Methods included from Ec2Base
#ami, #connection_string, #ec2_connection, #fetch_ami, #fetch_ec2_instance, #fetch_network_interfaces, #fetch_password_data, #fetch_region, #fetch_subnet, #find_name_tag, included, #is_image_windows?, #msg_pair, #normalize_server_data, #server_hashes, #validate_aws_config!, #vpc_mode?
Methods inherited from Chef::Knife
#aws_cred_file_location, #custom_warnings!, #find_server_platform, #iam_name_from_profile, #ini_parse
Instance Method Details
#run ⇒ Object
29 30 31 32 |
# File 'lib/chef/knife/ec2_flavor_list.rb', line 29 def run ui.error("knife ec2 flavor list has been deprecated as this functionality is not provided by the AWS API the previous implementation relied upon hardcoded values that were often incorrect. For an up to date list of instance types see https://www.ec2instances.info/") exit 1 end |